//
//  ViewController.m
//  SlidesDemo
//
//  Created by Alex Rezit on 23/02/2013.
//  Copyright (c) 2013 Seymour Dev. All rights reserved.
//

#import "PageView.h"
#import "ViewController.h"

@interface ViewController ()

@property (nonatomic, strong) NSMutableArray *visibleViews;
@property (nonatomic, strong) NSMutableSet *reusableViews;

@property (nonatomic, strong) UIScrollView *scrollView;

@property (nonatomic, assign) NSInteger currentPage;

#pragma mark - Log

- (void)outputViewReuseStatus;

#pragma mark - Page control

- (CGRect)frameForViewInPage:(NSUInteger)page;

- (void)addPagesInRange:(NSRange)pageRange;
- (void)removePagesInRange:(NSRange)pageRange;

- (PageView *)viewForPage:(NSUInteger)page;
- (PageView *)dequeueReusablePageView;

- (void)autoresizePages;

@end

@implementation ViewController

#pragma mark - Log

- (void)outputViewReuseStatus
{
    NSLog(@"Loaded pages: %d.", self.visibleViews.count);
    NSLog(@"Reusable pages: %d.", self.reusableViews.count);
}

#pragma mark - Page control

- (void)reloadPages
{
    [self removePagesInRange:NSMakeRange(0, NSIntegerMax)];
    NSUInteger currentPage = self.currentPage;
    NSUInteger beginPage = currentPage > 0 ? currentPage - 1 : 0;
    NSUInteger endPage = currentPage + 1;
    [self addPagesInRange:NSMakeRange(beginPage, endPage + 1 - beginPage)];
    
    [self outputViewReuseStatus];
}

- (CGRect)frameForViewInPage:(NSUInteger)page
{
    CGRect scrollViewBounds = self.scrollView.bounds;
    return CGRectMake(scrollViewBounds.size.width * page,
                      0,
                      scrollViewBounds.size.width,
                      scrollViewBounds.size.height);
}

- (void)addPagesInRange:(NSRange)pageRange
{
    NSRange maxRange = NSMakeRange(0, self.numberOfPages);
    pageRange = NSIntersectionRange(pageRange, maxRange);
    for (NSUInteger page = pageRange.location; page < NSMaxRange(pageRange); page++) {
        PageView *pageView = [self viewForPage:page];
        pageView.page = page;
        [self.visibleViews addObject:pageView];
        pageView.frame = [self frameForViewInPage:page];
        [self.scrollView addSubview:pageView];
    }
}

- (void)removePagesInRange:(NSRange)pageRange
{
    for (NSUInteger idx = 0; idx < self.visibleViews.count; idx++) {
        PageView *pageView = self.visibleViews[idx];
        if ([pageView isKindOfClass:[PageView class]] && NSLocationInRange(pageView.page, pageRange)) {
            [self.reusableViews addObject:pageView];
            [self.visibleViews removeObject:pageView];
            [pageView removeFromSuperview];
            idx--;
        }
    }
}

- (PageView *)viewForPage:(NSUInteger)page
{
    PageView *pageView = [self dequeueReusablePageView];
    if (!pageView) {
        pageView = [[[PageView alloc] init] autorelease];
    }
    
    // Custom initialization
    pageView.backgroundColor = [UIColor colorWithHue:(float)page/self.numberOfPages saturation:1.0 brightness:1.0 alpha:1.0];
    
    return pageView;
}

- (PageView *)dequeueReusablePageView
{
    PageView *pageView = [[self.reusableViews.anyObject retain] autorelease];
    if (pageView) {
        [self.reusableViews removeObject:pageView];
    }
    return pageView;
}

- (void)autoresizePages
{
    UIScrollView *scrollView = self.scrollView;
    scrollView.frame = self.view.bounds;
    CGRect scrollViewBounds = scrollView.bounds;
    scrollView.contentSize = CGSizeMake(scrollViewBounds.size.width * self.numberOfPages,
                                        scrollViewBounds.size.height);
    
    for (PageView *pageView in self.visibleViews) {
        if ([pageView isKindOfClass:[PageView class]]) {
            pageView.frame = [self frameForViewInPage:pageView.page];
        }
    }
}

- (void)scrollToPage:(NSInteger)page animated:(BOOL)animated
{
    CGRect scrollViewBounds = self.scrollView.bounds;
    CGPoint contentOffset = CGPointMake(scrollViewBounds.size.width * page, 0);
    [self.scrollView setContentOffset:contentOffset animated:animated];
}

#pragma mark - Getters and setters

- (void)setNumberOfPages:(NSUInteger)numberOfPages
{
    if (_numberOfPages != numberOfPages) {
        // Resize scroll view
        UIScrollView *scrollView = self.scrollView;
        CGRect scrollViewBounds = scrollView.bounds;
        scrollView.contentSize = CGSizeMake(scrollViewBounds.size.width * numberOfPages,
                                            scrollViewBounds.size.height);
        
        // Set value
        _numberOfPages = numberOfPages;
    }
}

- (void)setCurrentPage:(NSInteger)currentPage
{
    if (_currentPage != currentPage) {
        // Remove views
        NSInteger removeContentPageStart = 0;
        NSInteger removeContentPageEnd = 0;
        if (currentPage > _currentPage) {
            removeContentPageStart = MAX(_currentPage - 1, 0);
            removeContentPageEnd = currentPage - 2;
        } else {
            removeContentPageStart = currentPage + 2;
            removeContentPageEnd = MIN(_currentPage + 1, self.numberOfPages - 1);
        }
        [self removePagesInRange:NSMakeRange(removeContentPageStart,
                                             removeContentPageEnd + 1 - removeContentPageStart)];
        
        // Add views
        NSInteger addContentPageStart = 0;
        NSInteger addContentPageEnd = 0;
        if (currentPage > _currentPage) {
            addContentPageStart = MAX(currentPage - 1, _currentPage + 2);
            addContentPageEnd = MIN(currentPage + 1, self.numberOfPages - 1);
        } else {
            addContentPageStart = MAX(currentPage - 1, 0);
            addContentPageEnd = MIN(currentPage + 1, _currentPage - 2);
        }
        if (addContentPageStart < self.numberOfPages ||
            addContentPageEnd > -1) {
            [self addPagesInRange:NSMakeRange(addContentPageStart,
                                              addContentPageEnd + 1 - addContentPageStart)];
        }
        
        [self outputViewReuseStatus];
        
        // Set value
        _currentPage = currentPage;
    }
}

#pragma mark - Life cycle

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Set default value
        self.numberOfPages = 10;
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.visibleViews = [[[NSMutableArray alloc] init] autorelease];
    self.reusableViews = [[[NSMutableSet alloc] init] autorelease];
    
    // Initialize scroll view
    UIScrollView *scrollView = [[[UIScrollView alloc] init] autorelease];
    scrollView.delegate = self;
    scrollView.pagingEnabled = YES;
    scrollView.showsHorizontalScrollIndicator = NO;
    scrollView.showsVerticalScrollIndicator = NO;
    self.scrollView = scrollView;
    [self.view addSubview:scrollView];
    
    // Load data
    [self reloadPages];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self autoresizePages];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Auto rotate

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    NSInteger currentPage = self.currentPage;
    [super willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [self autoresizePages];
    [self scrollToPage:currentPage animated:NO];
}

#pragma mark - Scroll view delegate

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    CGRect bounds = scrollView.bounds;
    CGPoint contentOffset = scrollView.contentOffset;
    self.currentPage = (contentOffset.x + 0.5 * bounds.size.width) / bounds.size.width;
}

@end
